import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.StringBufferInputStream;
import java.util.ArrayList;
import java.util.List;

import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.WebTarget;

import org.glassfish.jersey.media.multipart.MultiPartFeature;

import EdFi.Rest.Api.BulkOperationExceptionsApi;
import EdFi.Rest.Api.BulkOperationsApi;
import EdFi.Rest.Api.UploadsApi;
import EdFi.Rest.Models.BulkOperation;
import EdFi.Rest.Models.BulkOperationException;
import EdFi.Rest.Models.Upload;
import EdFi.Rest.Models.UploadFile;
import EdFi.Rest.Sdk.BearerTokenAuthenticator;
import EdFi.Rest.Sdk.CertificateValidationDisabler;
import EdFi.Rest.Sdk.RestApiTokenRetriever;
import EdFi.Rest.Sdk.RestResponse;
import EdFi.Rest.Sdk.TokenRetriever;

public class BulkOperationApp {

	private static WebTarget target;  
	
	public static void main(String[] args) throws InterruptedException, IOException {
		// Trust all SSL certs -- needed unless signed SSL certificates are configured.
		CertificateValidationDisabler.disable();  // generated helper class

		// Oauth configuration
		String oauthUrl = "https://tn-rest-admin-production.cloudapp.net";
		String clientKey = "1VSqxR1M9r4F";
		String clientSecret = "UxCi8RT7WTSRaFX8EC73In1h";

		// TokenRetriever makes the oauth calls
		TokenRetriever tokenRetriever = new RestApiTokenRetriever(oauthUrl, clientKey, clientSecret);

		// JAX-RS dependency, installed via Maven
		ClientBuilder clientBuilder = ClientBuilder.newBuilder();
		Client client = clientBuilder.register(MultiPartFeature.class).build().register(new BearerTokenAuthenticator(tokenRetriever));
		
		target = client.target("https://tn-rest-production.cloudapp.net/api/v1.0");

		BulkOperationsApi bulkOperationsApi = new BulkOperationsApi(target);
		BulkOperation bulkOperation = new BulkOperation();
		List<UploadFile> uploadFiles = new ArrayList<UploadFile>();
		UploadFile uploadFile = new UploadFile();
		String filePath = "D:\\test.xml";
		File file = new File(filePath);
		int fileLength = (int)file.length();
		
		uploadFile.setInterchangeType("EducationOrganization");
		uploadFile.setFormat("text/xml");
		uploadFile.setSize(fileLength);
		
		uploadFiles.add(uploadFile);
		bulkOperation.setUploadFiles(uploadFiles);

		// Create the bulk operation by posting the metadata.
		RestResponse<BulkOperation> response = bulkOperationsApi.postBulkOperations(bulkOperation);
		
		bulkOperation = response.readEntity();
		
		uploadFile = bulkOperation.getUploadFiles().get(0);

		System.out.println("Bulk Operation Id " + bulkOperation.getId());
		System.out.println("Upload File Id " + uploadFile.getId());
				
		// Upload the file bytes.
		UploadsApi uploadsApi = new UploadsApi(target);

		// Add bytes to be uploaded.
		Upload upload = new Upload();
		byte[] fileBytes = new byte[fileLength];

		upload.setId(uploadFile.getId());
		upload.setSize(fileLength);
		// upload.setSize(36);
		upload.setOffset(0);
		
		FileInputStream fileInputStream = new FileInputStream(filePath);
		
		fileInputStream.read(fileBytes);
		
    	// upload.setFileStream(new FileInputStream(filePath));
    	// upload.setFileStream(new ByteArrayInputStream("Test data".getBytes()));
		
    	upload.setFileBytes(fileBytes);
		
		// Upload the bytes.
    	RestResponse uploadResponse = uploadsApi.postUploads(upload);	

    	System.out.println(uploadResponse.getStatusInfo().getStatusCode());
    	System.out.println(uploadResponse.getStatusInfo().getReasonPhrase());
    	
    	upload.setFileBytes(null);
  	
		// Commit the upload.
    	RestResponse commitResponse = uploadsApi.commitUploads(upload);
		
    	System.out.println(commitResponse.getStatusInfo().getStatusCode());
    	System.out.println(commitResponse.getStatusInfo().getReasonPhrase());
		
		// Wait and then check for errors.
		Thread.sleep(5000);
		
		response = bulkOperationsApi.getBulkOperationsById(bulkOperation.getId());
		
		bulkOperation = response.readEntity();
		
		System.out.println(bulkOperation.getStatus());
		
		BulkOperationExceptionsApi bulkOperationExceptionsApi = new BulkOperationExceptionsApi(target);
		
		for (UploadFile returnedUploadFile : bulkOperation.getUploadFiles())
        {
			// Check the status of the files.
        	System.out.format("File - %s %s %s\n", returnedUploadFile.getId(), returnedUploadFile.getInterchangeType(), returnedUploadFile.getStatus());

        	RestResponse<List<BulkOperationException>> bulkOperationExceptions = bulkOperationExceptionsApi.getBulkOperationExceptions(bulkOperation.getId(), returnedUploadFile.getId(), null, null);
        	
    		for(BulkOperationException bulkOperationException : bulkOperationExceptions.readEntity())
    		{
            	System.out.format("Exception - %s %s %s\n", returnedUploadFile.getId(), returnedUploadFile.getInterchangeType(), returnedUploadFile.getStatus());
    		}
        }
	}
}