package EdFi.Rest.Api;

import EdFi.Rest.Sdk.RestResponse;
import EdFi.Rest.Models.Identity;
import EdFi.Rest.Models.Student;

import java.util.*;

import javax.ws.rs.client.Entity;
import javax.ws.rs.client.Invocation.Builder;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

public class IdentitiesApi {
    private final WebTarget baseTarget;
  
    public IdentitiesApi(WebTarget baseTarget) {
        this.baseTarget = baseTarget;
    }

    // This signature should be setup as follows. This will change on a subsequent API update.
    // public IestResponse<ArrayList<Identity>> getIdentitiesByExample(Identity identity)
    public RestResponse<List<Identity>> getIdentitiesByExample(Identity identity) {
        WebTarget target = baseTarget.path("/identities");

        target = target.queryParam("familyNames", identity.getFamilyNames());
        target = target.queryParam("givenNames", identity.getGivenNames());

        if (identity.getBirthGender() != null)
        	target = target.queryParam("birthGender", identity.getBirthGender());

        if (identity.getBirthDate() != null)
        	target = target.queryParam("birthDate", identity.getBirthDate());
        
        Builder builder = target.request(MediaType.APPLICATION_JSON);
        return new RestResponse<List<Identity>>(builder.method("GET")) {};
        }

    public RestResponse<Identity> postIdentity(Identity identity) {
        WebTarget target = baseTarget.path("/identities");

        // verify required params are set
        if (identity == null)
        {
            throw new IllegalArgumentException("API method call is missing required parameters");
        }
        Builder builder = target.request(MediaType.APPLICATION_JSON);
        return new RestResponse<Identity>(builder.method("POST", Entity.json(identity))) {};
	}    
  
    public RestResponse<Identity> getIdentitiesById(String uniqueId) {
        WebTarget target = baseTarget.path("/identities/{id}");

        // verify required params are set
        if (uniqueId == null ) {
            throw new IllegalArgumentException("API method call is missing required parameters");
        }
        target = target.resolveTemplate("id", uniqueId);
        Builder builder = target.request(MediaType.APPLICATION_JSON);
        return new RestResponse<Identity>(builder.method("GET")) {};
    }
}
